import { ChangeDetectorRef, Component, EventEmitter, Input, Output } from '@angular/core';
import { ToastrService } from 'ngx-toastr';
import { AppraisalRoundModel, MyAppraisalRoundModel } from 'src/app/shared/model/appraisal-aps.model';
import { AppraisalFormModel } from 'src/app/shared/model/appraisal-form.model';
import { AppraisalModel, MyAppraisalModel } from 'src/app/shared/model/appraisal.model';
import { EvaluationCycleModel, MyEvaluationCycleModel } from 'src/app/shared/model/evaluation-cycle.model';
import { EvaluationAssessmentService } from 'src/app/shared/services/evaluation-assessment.service';
import { EvaluationCycleService } from 'src/app/shared/services/evaluation-cycle.service';


@Component({
  selector: 'app-edit-evaluation-form',
  templateUrl: './edit-evaluation-form.component.html',
  styleUrls: ['./edit-evaluation-form.component.scss']
})
export class EditEvaluationFormComponent {
  @Input() pathTitle = ['การประเมินจัดการประสิทธิภาพ', 'รอบการประเมิน', 'กำหนดฟอร์มเอกสาร', 'เเก้ไขฟอร์มตามรอบการประเมิน'];
  @Output() sendPathTitle: EventEmitter<string[]> = new EventEmitter<string[]>();
  @Output() sendGroupShow: EventEmitter<string> = new EventEmitter<string>();
  @Input() evaluationRoundId = ''

  currentPage = 1
  page = Array.from({ length: 1 }, (_, i) => i + 1);
  search = ""

  assessmentList: AppraisalModel[] = []

  evaluationRound:EvaluationCycleModel = new MyEvaluationCycleModel()

  isChecked: boolean = false; 
  
  apsassessy = ""
  empassessmentList: AppraisalModel[] = []
  searchEmp = ""
  
  empassessmentRoundList: AppraisalRoundModel[] = []
  empassessmentRoundListCheckbox: {checkbox: boolean, item: AppraisalRoundModel}[] = []
  currentPageRound = 1
  pageRound = Array.from({ length: 1 }, (_, i) => i + 1);
  searchRound = ""
  selectEmp:AppraisalModel = new MyAppraisalModel() 
  numDataListChecked = 0
  msg = ""
  typeForm = ""

  selectData: AppraisalRoundModel = new MyAppraisalRoundModel()
  constructor(private toastr: ToastrService,
    private cdr: ChangeDetectorRef,
    private evaluationAssessmentService: EvaluationAssessmentService,
    private evaluationCycleService: EvaluationCycleService
  ) {

  }

  ngOnInit(): void {

  }
  ngOnChanges(): void {
    setTimeout(() => {
      document.getElementById('form-pm-emp')?.classList.add("hidden")
      document.getElementById('form-assessment')?.classList.add("hidden")
      document.getElementById('modal-alert')?.classList.add("hidden")
    }, 10);
    if (this.evaluationRoundId) {
      this.getAssessmentId();
      this.getAssessmentList();
    }

  }
  getAssessmentId() {
    this.evaluationCycleService.getById(this.evaluationRoundId!).subscribe({
      next: response => {
        this.evaluationRound = response
        this.cdr.detectChanges();
      },
      error: err => {
        console.error('Error fetching evaluation cycles:', err);
      }
    });

  }
  getAssessmentList() {
    this.evaluationAssessmentService.getById(this.evaluationRoundId!).subscribe({
      next: response => {
        this.assessmentList = response
        this.cdr.detectChanges();
      },
      error: err => {
        console.error('Error fetching evaluation cycles:', err);
      }
    });
  }
  assessmentListFilter() {
    return this.assessmentList.filter(x =>
      x.apsassessy.fname.toLowerCase().includes(this.search.toLowerCase()) ||
      x.apsassessy.lname.toLowerCase().includes(this.search.toLowerCase()) ||
      x.apsassessy.efname.toLowerCase().includes(this.search.toLowerCase()) ||
      x.apsassessy.elname.toLowerCase().includes(this.search.toLowerCase()) ||
      x.masfromEvaluationRound.evaluationRoundId.toLowerCase().includes(this.search.toLowerCase()) ||
      x.apsassessy.employeeId.toLowerCase().includes(this.search.toLowerCase())
    )
  }
  searchChange() {
    this.currentPage = 1;
    const filteredData = this.assessmentListFilter();
    this.page = Array.from({ length: Math.ceil(filteredData.length / 10) }, (_, i) => i + 1);
  }

  openModalClass(id: string,type?:string) {
    this.typeForm = type??""
    if(id == 'modal-alert'){
      if(type == 'deleteForm'){
        this.msg = 'ยืนยันการลบรายการ'
      }else if(type == 'addForm'){
        this.msg = 'ยืนยันการบันทึกรายการ'
      }
    }
    if(id){
      document.getElementById(id)?.classList.add('open');
      document.getElementById(id)?.classList.remove('hidden');
      setTimeout(() => {
        document.getElementById(id)?.setAttribute('aria-overlay', 'false')
        
      }, 10);
    }
   
    this.isChecked = false;
    this.numDataListChecked = 0;
  }

  closeModalClass(id: string) {
    const modal = document.getElementById(id);
    console.log("🚀 ~ EditEvaluationFormComponent ~ closeModalClass ~ modal:", modal)
    if (id) {
      modal?.classList.add("hidden"); // ซ่อน Modal
    }
  }
 


  saveAssessmentList() {
    let body = this.assessmentList.filter(x => x.active == true)
    if (body.length > 0) {
      this.evaluationAssessmentService.postList(body).subscribe({
        next: response => {
          if (response.success) {
            this.showAlert(response.message, 'success')
            this.getAssessmentList();
          } else {
          }
        }
      })
    } else {
      this.showAlert('ไม่พบรายการ', 'error')
    }
  }

  openAddRoundModel(evaluationRoundId:string,apsassessy:string,item:AppraisalModel,model:string) {
    this.selectEmp = item
      this.openModalClass(model);
      this.evaluationRoundId = evaluationRoundId
      this.apsassessy = apsassessy
      this.getApsassessy()
      this.getApsassessyMini();
    }
    
  getApsassessy(){
    this.evaluationAssessmentService.getApsassessy(this.evaluationRoundId,this.apsassessy).subscribe({
      next: response => {
        this.empassessmentList = response
        this.cdr.detectChanges();
      },
      error: err => {
        console.error('Error fetching evaluation cycles:', err);
    }
  })
  }
  getApsassessyMini(){
    this.evaluationAssessmentService.getApsassessyMini(this.evaluationRoundId,this.apsassessy).subscribe({
      next: response => {
        this.empassessmentRoundList = response
        this.empassessmentRoundListCheckbox = this.empassessmentRoundList.map(item => ({checkbox: false, item}))
        this.cdr.detectChanges();
      },
      error: err => {
        console.error('Error fetching evaluation cycles:', err);
    }
  })
  }
  empassessmentListFilter() {
    return this.empassessmentRoundListCheckbox.filter(x =>
      x.item.groupAssessment1?.competencyIndicatorsCourses1Mini?.competencyIndicatorsCourses0?.competencyTopic?.tdesc?.toLowerCase()?.includes(this.searchRound?.toLowerCase() ?? "")||
      x.item.groupAssessment1?.competencyIndicatorsCourses1Mini?.competencyIndicatorsCourses0?.competencyTopic?.edesc?.toLowerCase()?.includes(this.searchRound?.toLowerCase() ?? "") ||
      x.item.groupAssessment1?.competencyIndicatorsCourses1Mini?.competencyIndicatorsCourses0?.competencyTopic?.competencyTopicId?.toLowerCase()?.includes(this.searchRound?.toLowerCase() ?? ""))
  }
  searchEmpassessmentListFilterChange() {
    this.currentPageRound = 1;
    const filteredData = this.empassessmentListFilter();
    this.pageRound = Array.from({ length: Math.ceil(filteredData.length / 10) }, (_, i) => i + 1);
  }
  toggleCheckbox() {
    this.isChecked = !this.isChecked; // สลับสถานะ
    this.empassessmentRoundListCheckbox.map(item => item.checkbox = this.isChecked);
    this.checkempassessmentRoundListCheckbox();
  }

  checkempassessmentRoundListCheckbox(){
    this.numDataListChecked = this.empassessmentRoundListCheckbox.filter(x => x.checkbox).length
    this.isChecked = this.numDataListChecked ? true : false

  }
  deleteFormOne(item:AppraisalRoundModel){
    this.selectData = item
  }

  deleteAssessment() {
    let body = this.empassessmentRoundListCheckbox.filter(x => x.checkbox).map(x => x.item)
    if(this.typeForm=='deleteFormOne'){
      body = [this.selectData]
    }
    if( body.length == 0){
      this.showAlert('กรุณาเลือกรายการ', 'error')
      return
    }
    this.evaluationAssessmentService.deletemMini(body,this.selectEmp.apsassessy.employeeId,this.selectEmp.masfromEvaluationRound.evaluationRoundId).subscribe({
      next: response => {
        if (response.success) {
          this.showAlert(response.message, 'success')
          this.getApsassessyMini();
        } else {
          this.showAlert(response.message, 'error')
        }  
      }
    })
  }

  showAlert(text: string, type: 'success' | 'error') {
    this.toastr[type](text, 'แจ้งเตือน', {
      timeOut: 3000,
      positionClass: 'toast-top-right',
    });
  }







}
